/*
 *      netwindow.c
 *      
 *      Copyright 2009 - 2010 Alex <alex@linuxonly.ru>
 *      
 *      This program is free software: you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 3 of the License, or
 *      (at your option) any later version.
 *      
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *      
 *      You should have received a copy of the GNU General Public License
 *      along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdio.h>
#include <gtk/gtk.h>
#include <glib/gi18n.h>
#include <string.h>
#include "settings.h"
#include "mainwindow.h"
//#include "gost.h"

struct snetwindow {
	//netconnect setup window
	gboolean visible;
	GtkWidget *win;
	gchar *title;
	GtkWidget *caption;
	GtkWidget *close;
	GtkWidget *apply;
	GtkWidget *ip;
	GtkWidget *passwd;
	GRegex *ipregex;
	gboolean regex_lock;
};
struct snetwindow netwindow;

gint netwindow_close(GtkWidget *widget, GdkEvent *event, gpointer data);
gint netwindow_apply(GtkWidget *widget, GdkEvent *event, gpointer data);
void netwindow_validate_regex(GtkEditable *editable, gpointer data);


gint netwindow_close(GtkWidget *widget, GdkEvent *event, gpointer data)
{
    netwindow.visible = FALSE;
	if (netwindow.regex_lock) {
		g_regex_unref(netwindow.ipregex);
		netwindow.regex_lock = FALSE;
	}
	gtk_widget_destroy(netwindow.win);
    return FALSE;
}

gint netwindow_apply(GtkWidget *widget, GdkEvent *event, gpointer data)
{
	gchar *address, *password;
	
	address = g_strdup(gtk_entry_get_text(GTK_ENTRY(netwindow.ip)));
	settings_append_string("network_server_address", address);
	password = g_strdup(gtk_entry_get_text(GTK_ENTRY(netwindow.passwd)));
	settings_append_string("network_server_password", password);
	settings_flush();
	
	netwindow_close(NULL, NULL, NULL);
	
	mainwindow_network_start(address, password);	
	
	return FALSE;
}


void netwindow_validate_regex(GtkEditable *editable, gpointer data)
{
	if (strlen(gtk_entry_get_text(GTK_ENTRY(editable))) == 0) {
		gtk_widget_set_sensitive(netwindow.apply, FALSE);
	} else {
		gtk_widget_set_sensitive(netwindow.apply, TRUE);
	}
}

gboolean netwindow_create(struct _netgate *netgate)
{
    GtkWidget *vbox, *hbox;
	GtkWidget *align;
	GtkWidget *gentable;
	GtkWidget *ip_l;
	GtkWidget *passwd_l;
	GtkWidget *sep;
	PangoFontDescription *fds;
	GdkColor color;
	GtkWidget *chbox;
	GtkWidget *evbox;
	GtkWidget *icon;

	
	
	if (!netwindow.visible) {
	  netwindow.ipregex = g_regex_new("^(\\d|[1-9]\\d|1\\d\\d|2[0-4]\\d|25[0-5])(\\.(\\d|[1-9]\\d|1\\d\\d|2[0-4]\\d|25[0-5])){3}$", G_REGEX_OPTIMIZE, G_REGEX_MATCH_NOTEMPTY, NULL);
	  netwindow.visible = TRUE;  
	  netwindow.regex_lock = TRUE;
	  netwindow.win = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	  gtk_window_set_modal(GTK_WINDOW(netwindow.win), TRUE);
	  gtk_window_set_resizable(GTK_WINDOW(netwindow.win), FALSE);
	  netwindow.title = _("Network signal status");
	  gtk_window_set_position(GTK_WINDOW(netwindow.win), GTK_WIN_POS_CENTER);
	  gtk_window_set_title(GTK_WINDOW(netwindow.win), netwindow.title);
	  gtk_widget_set_size_request(netwindow.win, 400, 200);
	  gtk_window_set_default_size(GTK_WINDOW(netwindow.win), 400, 200);
	  
	  
	  vbox = gtk_vbox_new(FALSE, 0);
	  
	  fds = pango_font_description_from_string("14");
	  pango_font_description_set_weight(fds,PANGO_WEIGHT_BOLD);
	  netwindow.caption = gtk_label_new(_("Network signal status"));
	  gtk_widget_modify_font(netwindow.caption, fds);
	  pango_font_description_free(fds);
	  evbox = gtk_event_box_new();
	  gtk_event_box_set_visible_window(GTK_EVENT_BOX(evbox),TRUE);
	  gdk_color_parse("white", &color);
	  gtk_widget_modify_bg(evbox, GTK_STATE_NORMAL, &color);
	  chbox = gtk_hbox_new(FALSE,0);
	  gtk_container_set_border_width(GTK_CONTAINER(chbox), 5);
	  icon = gtk_image_new_from_stock(GTK_STOCK_NETWORK, GTK_ICON_SIZE_DIALOG);
	  gtk_box_pack_start(GTK_BOX(chbox),icon,FALSE,FALSE,15);
	  align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align),netwindow.caption);
	  gtk_box_pack_start(GTK_BOX(chbox),align,FALSE,FALSE,0);
	  gtk_container_add(GTK_CONTAINER(evbox),chbox);
	  
	  gtk_box_pack_start(GTK_BOX(vbox),evbox,FALSE,FALSE,0);
	  
	  gentable = gtk_table_new(2,2,FALSE);
	  	  
	  ip_l = gtk_label_new(" ");
	  gtk_label_set_markup(GTK_LABEL(ip_l), _("<b>Server address:</b>"));
	  align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), ip_l);
	  gtk_table_attach_defaults(GTK_TABLE(gentable),align,0,1,0,1);
	  
	  netwindow.ip = gtk_entry_new();
	  gtk_entry_set_text(GTK_ENTRY(netwindow.ip), settings_read_string("network_server_address", "127.0.0.1"));
	  align = gtk_alignment_new(0.0, 0.5, 1.0, 1.0);
	  gtk_container_add(GTK_CONTAINER(align), netwindow.ip);
	  gtk_table_attach_defaults(GTK_TABLE(gentable),align,1,2,0,1);
	  
	  passwd_l = gtk_label_new(" ");
	  gtk_label_set_markup(GTK_LABEL(passwd_l), _("<b>Password:</b>"));
	  align = gtk_alignment_new(0.0, 0.5, 0.0, 0.0);
	  gtk_container_add(GTK_CONTAINER(align), passwd_l);
	  gtk_table_attach_defaults(GTK_TABLE(gentable),align,0,1,1,2);
	  
	  netwindow.passwd = gtk_entry_new();
	  gtk_entry_set_visibility(GTK_ENTRY(netwindow.passwd), FALSE);
	  gtk_entry_set_text(GTK_ENTRY(netwindow.passwd), settings_read_string("network_server_password", "defaultpassword"));
	  align = gtk_alignment_new(0.0, 0.5, 1.0, 1.0);
	  gtk_container_add(GTK_CONTAINER(align), netwindow.passwd);
	  gtk_table_attach_defaults(GTK_TABLE(gentable),align,1,2,1,2);
	  
	  gtk_container_set_border_width(GTK_CONTAINER(gentable),10);

	  gtk_box_pack_start(GTK_BOX(vbox), gentable, TRUE, TRUE, 0);
	  
	  netwindow.close = gtk_button_new_from_stock(GTK_STOCK_CLOSE);
	  netwindow.apply = gtk_button_new_from_stock(GTK_STOCK_APPLY);
	  	  
	  hbox = gtk_hbox_new(FALSE, 0);
	  gtk_box_pack_end(GTK_BOX(hbox), netwindow.apply, FALSE, FALSE, 5);
	  gtk_box_pack_end(GTK_BOX(hbox), netwindow.close, FALSE, FALSE, 5);
	  
	  
	  gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 5);

      g_signal_connect(G_OBJECT(netwindow.win), "destroy",
                       G_CALLBACK(netwindow_close), NULL);
	  
	  g_signal_connect(G_OBJECT(netwindow.close), "clicked",
                       G_CALLBACK(netwindow_close), NULL);
					   
	  g_signal_connect(G_OBJECT(netwindow.apply), "clicked",
                       G_CALLBACK(netwindow_apply), NULL);
					   
	  g_signal_connect(G_OBJECT(netwindow.ip), "changed",
                       G_CALLBACK(netwindow_validate_regex), NULL);
					   

 	  gtk_container_add(GTK_CONTAINER(netwindow.win),vbox);
	  gtk_widget_show_all(netwindow.win);
	  
	  gtk_window_set_focus(GTK_WINDOW(netwindow.win), netwindow.apply);
	  
  } else {
	  
	  gtk_window_present(GTK_WINDOW(netwindow.win));
	  
  }
	  
      return TRUE;
}
